Documentation
¶
Overview ¶
Package turn implements RFC 5766 Traversal Using Relays around NAT.
Index ¶
Examples ¶
Constants ¶
const ( // DefaultPort for TURN is same as STUN. DefaultPort = stun.DefaultPort // DefaultTLSPort is for TURN over TLS and is same as STUN. DefaultTLSPort = stun.DefaultTLSPort )
Default ports for TURN from RFC 5766 Section 4.
const DefaultLifetime = time.Minute * 10
DefaultLifetime in RFC 5766 is 10 minutes.
Variables ¶
var ( // AllocateRequest is shorthand for allocation request message type. AllocateRequest = stun.NewType(stun.MethodAllocate, stun.ClassRequest) // CreatePermissionRequest is shorthand for create permission request type. CreatePermissionRequest = stun.NewType(stun.MethodCreatePermission, stun.ClassRequest) // SendIndication is shorthand for send indication message type. SendIndication = stun.NewType(stun.MethodSend, stun.ClassIndication) // RefreshRequest is shorthand for refresh request message type. RefreshRequest = stun.NewType(stun.MethodRefresh, stun.ClassRequest) )
var RequestedTransportUDP stun.Setter = RequestedTransport{ Protocol: ProtoUDP, }
RequestedTransportUDP is setter for requested transport attribute with value ProtoUDP (17).
var ZeroLifetime stun.Setter = Lifetime{}
ZeroLifetime is shorthand for setting zero lifetime that indicates to close allocation.
Functions ¶
This section is empty.
Types ¶
type BadAttrLength ¶
BadAttrLength means that length for attribute is invalid.
func (BadAttrLength) Error ¶
func (e BadAttrLength) Error() string
type ChannelNumber ¶
type ChannelNumber int // encoded as uint16
ChannelNumber represents CHANNEL-NUMBER attribute.
The CHANNEL-NUMBER attribute contains the number of the channel.
func (ChannelNumber) AddTo ¶
func (n ChannelNumber) AddTo(m *stun.Message) error
AddTo adds CHANNEL-NUMBER to message.
func (*ChannelNumber) GetFrom ¶
func (n *ChannelNumber) GetFrom(m *stun.Message) error
GetFrom decodes CHANNEL-NUMBER from message.
func (ChannelNumber) String ¶
func (n ChannelNumber) String() string
type Data ¶
type Data []byte
Data represents DATA attribute.
The DATA attribute is present in all Send and Data indications. The value portion of this attribute is variable length and consists of the application data (that is, the data that would immediately follow the UDP header if the data was been sent directly between the client and the peer).
type DontFragmentAttr ¶
type DontFragmentAttr bool
DontFragmentAttr represents DONT-FRAGMENT attribute.
var DontFragment DontFragmentAttr = true
DontFragment is shorthand for DontFragmentAttr.
type EvenPort ¶
type EvenPort struct {
// ReservePort means that the server is requested to reserve
// the next-higher port number (on the same IP address)
// for a subsequent allocation.
ReservePort bool
}
EvenPort represents EVEN-PORT attribute.
This attribute allows the client to request that the port in the relayed transport address be even, and (optionally) that the server reserve the next-higher port number.
type Lifetime ¶
Lifetime represents LIFETIME attribute.
The LIFETIME attribute represents the duration for which the server will maintain an allocation in the absence of a refresh. The value portion of this attribute is 4-bytes long and consists of a 32-bit unsigned integral value representing the number of seconds remaining until expiration.
Example ¶
// Encoding lifetime to message.
m := new(stun.Message)
Lifetime{time.Minute}.AddTo(m)
m.WriteHeader()
// Decoding message.
mDec := new(stun.Message)
if _, err := m.WriteTo(mDec); err != nil {
panic(err)
}
// Decoding lifetime from message.
l := Lifetime{}
l.GetFrom(m)
fmt.Println("Decoded:", l)
Output: Decoded: 1m0s
type PeerAddress ¶
PeerAddress implements XOR-PEER-ADDRESS attribute.
The XOR-PEER-ADDRESS specifies the address and port of the peer as seen from the TURN server. (For example, the peer's server-reflexive transport address if the peer is behind a NAT.)
func (PeerAddress) AddTo ¶
func (a PeerAddress) AddTo(m *stun.Message) error
AddTo adds XOR-PEER-ADDRESS to message.
func (*PeerAddress) GetFrom ¶
func (a *PeerAddress) GetFrom(m *stun.Message) error
GetFrom decodes XOR-PEER-ADDRESS from message.
func (PeerAddress) String ¶
func (a PeerAddress) String() string
type Protocol ¶
type Protocol byte
Protocol is IANA assigned protocol number.
const ( // ProtoUDP is IANA assigned protocol number for UDP. ProtoUDP Protocol = 17 )
type RelayedAddress ¶
RelayedAddress implements XOR-RELAYED-ADDRESS attribute.
The XOR-PEER-ADDRESS specifies the address and port of the peer as seen from the TURN server. (For example, the peer's server-reflexive transport address if the peer is behind a NAT.)
func (RelayedAddress) AddTo ¶
func (a RelayedAddress) AddTo(m *stun.Message) error
AddTo adds XOR-PEER-ADDRESS to message.
func (*RelayedAddress) GetFrom ¶
func (a *RelayedAddress) GetFrom(m *stun.Message) error
GetFrom decodes XOR-PEER-ADDRESS from message.
func (RelayedAddress) String ¶
func (a RelayedAddress) String() string
type RequestedTransport ¶
type RequestedTransport struct {
Protocol Protocol
}
RequestedTransport represents REQUESTED-TRANSPORT attribute.
This attribute is used by the client to request a specific transport protocol for the allocated transport address. RFC 5766 only allows the use of codepoint 17 (User Datagram Protocol).
func (RequestedTransport) AddTo ¶
func (t RequestedTransport) AddTo(m *stun.Message) error
AddTo adds REQUESTED-TRANSPORT to message.
func (*RequestedTransport) GetFrom ¶
func (t *RequestedTransport) GetFrom(m *stun.Message) error
GetFrom decodes REQUESTED-TRANSPORT from message.
func (RequestedTransport) String ¶
func (t RequestedTransport) String() string
type ReservationToken ¶
type ReservationToken []byte
ReservationToken represents RESERVATION-TOKEN attribute.
The RESERVATION-TOKEN attribute contains a token that uniquely identifies a relayed transport address being held in reserve by the server. The server includes this attribute in a success response to tell the client about the token, and the client includes this attribute in a subsequent Allocate request to request the server use that relayed transport address for the allocation.